==> FK-2_8_9.tar.bz2: FK-2_8_9/src/Surface.cxx <==
/****************************************************************************
 *
 *	Copyright (c) 1999-2009, Fine Kernel Project, All rights reserved.
 *
 *	Redistribution and use in source and binary forms,
 *	with or without modification, are permitted provided that the
 *	following conditions are met:
 *
 *		- Redistributions of source code must retain the above
 *			copyright notice, this list of conditions and the
 *			following disclaimer.
 *
 *		- Redistributions in binary form must reproduce the above
 *			copyright notice, this list of conditions and the
 *			following disclaimer in the documentation and/or
 *			other materials provided with the distribution.
 *
 *		- Neither the name of the copyright holders nor the names
 *			of its contributors may be used to endorse or promote
 *			products derived from this software without specific
 *			prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *	"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *	LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *	FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *	COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *	STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 *	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *	POSSIBILITY OF SUCH DAMAGE. 
 *
 ****************************************************************************/
/****************************************************************************
 *
 *	Copyright (c) 1999-2009, Fine Kernel Project, All rights reserved.
 *
 *	本ソフトウェアおよびソースコードのライセンスは、基本的に
 *	「修正 BSD ライセンス」に従います。以下にその詳細を記します。
 *
 *	ソースコード形式かバイナリ形式か、変更するかしないかを問わず、
 *	以下の条件を満たす場合に限り、再頒布および使用が許可されます。
 *
 *	- ソースコードを再頒布する場合、上記の著作権表示、本条件一覧、
 *		および下記免責条項を含めること。
 *
 *	- バイナリ形式で再頒布する場合、頒布物に付属のドキュメント等の
 *		資料に、上記の著作権表示、本条件一覧、および下記免責条項を
 *		含めること。
 *
 *	- 書面による特別の許可なしに、本ソフトウェアから派生した製品の
 *		宣伝または販売促進に、本ソフトウェアの著作権者の名前または
 *		コントリビューターの名前を使用してはならない。
 *
 *	本ソフトウェアは、著作権者およびコントリビューターによって「現
 *	状のまま」提供されており、明示黙示を問わず、商業的な使用可能性、
 *	および特定の目的に対する適合性に関す暗黙の保証も含め、またそれ
 *	に限定されない、いかなる保証もないものとします。著作権者もコン
 *	トリビューターも、事由のいかんを問わず、損害発生の原因いかんを
 *	問わず、かつ責任の根拠が契約であるか厳格責任であるか(過失その
 *	他の)不法行為であるかを問わず、仮にそのような損害が発生する可
 *	能性を知らされていたとしても、本ソフトウェアの使用によって発生
 *	した(代替品または代用サービスの調達、使用の喪失、データの喪失、
 *	利益の喪失、業務の中断も含め、またそれに限定されない)直接損害、
 *	間接損害、偶発的な損害、特別損害、懲罰的損害、または結果損害に
 *	ついて、一切責任を負わないものとします。
 *
 ****************************************************************************/
#include <FK/Surface.h>

using namespace std;

fk_Surface::fk_Surface(void)
{
	div = -1;
	changeFlg = true;
	posCache.clear();
	normCache.clear();

	return;
}

fk_Surface::~fk_Surface(void)
{
	return;
}

vector<fk_Vector> * fk_Surface::getPosCache(void)
{
	return &posCache;
}

vector<fk_Vector> * fk_Surface::getNormCache(void)
{
	return &normCache;
}

void fk_Surface::setDiv(int argDiv)
{
	if(argDiv < 1) return;

	if(argDiv != div) {
		changeFlg = true;
		div = argDiv;
	}
	return;
}

int fk_Surface::getDiv(void)
{
	return div;
}

fk_Vector fk_Surface::norm(double argU, double argV)
{
	fk_Vector		uVec, vVec, nVec;

	uVec = uDeriv(argU, argV);
	vVec = vDeriv(argU, argV);
	nVec = uVec ^ vVec;
	if(nVec.normalize() == false) {
		nVec.set(0.0, 0.0, 0.0);
	}

	return nVec;
}

void fk_Surface::makeCache(bool argSmoothFlg)
{
	if(div <= 0) return;

	makePosCache();
	makeNormCache(argSmoothFlg);
	changeFlg = false;
	return;
}

void fk_Surface::makePosCache(void)
{
	double		u, v;
	int			i, j;

	if(changeFlg == false) return;

	posCache.clear();

	for(i = 0; i <= div; i++) {
		v = double(i)/double(div);
		for(j = 0; j <= div; j++) {
			u = double(j)/double(div);
			posCache.push_back(pos(u, v));
		}
	}

	return;
}

void fk_Surface::makeNormCache(bool argSmoothFlg)
{
	double		u, v;
	int			i, j;
	fk_Vector	tmpV1, tmpV2, tmpV3, tmpNorm;

	if(changeFlg == false && smoothFlg == argSmoothFlg) return;

	normCache.clear();

	if(smoothFlg == true) {
		for(i = 0; i <= div; i++) {
			v = double(i)/double(div);
			for(j = 0; j <= div; j++) {
				u = double(j)/double(div);
				normCache.push_back(norm(u, v));
			}
		}
	} else {
		for(i = 0; i < div; i++) {
			for(j = 0; j < div; j++) {
				tmpV1 = posCache[i*(div+1)+j];
				tmpV2 = posCache[i*(div+1)+j+1];
				tmpV3 = posCache[(i+1)*(div+1)+j];
				tmpNorm = (tmpV2 - tmpV1) ^ (tmpV3 - tmpV1);
				tmpNorm.normalize();
				normCache.push_back(tmpNorm);
			}
		}
	}

	return;
}

==> FK-2_8_10.tar.bz2: FK-2_8_10/src/Surface.cxx <==
/****************************************************************************
 *
 *	Copyright (c) 1999-2011, Fine Kernel Project, All rights reserved.
 *
 *	Redistribution and use in source and binary forms,
 *	with or without modification, are permitted provided that the
 *	following conditions are met:
 *
 *		- Redistributions of source code must retain the above
 *			copyright notice, this list of conditions and the
 *			following disclaimer.
 *
 *		- Redistributions in binary form must reproduce the above
 *			copyright notice, this list of conditions and the
 *			following disclaimer in the documentation and/or
 *			other materials provided with the distribution.
 *
 *		- Neither the name of the copyright holders nor the names
 *			of its contributors may be used to endorse or promote
 *			products derived from this software without specific
 *			prior written permission.
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *	"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *	LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *	FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *	COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *	HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *	STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 *	IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *	POSSIBILITY OF SUCH DAMAGE. 
 *
 ****************************************************************************/
/****************************************************************************
 *
 *	Copyright (c) 1999-2011, Fine Kernel Project, All rights reserved.
 *
 *	本ソフトウェアおよびソースコードのライセンスは、基本的に
 *	「修正 BSD ライセンス」に従います。以下にその詳細を記します。
 *
 *	ソースコード形式かバイナリ形式か、変更するかしないかを問わず、
 *	以下の条件を満たす場合に限り、再頒布および使用が許可されます。
 *
 *	- ソースコードを再頒布する場合、上記の著作権表示、本条件一覧、
 *		および下記免責条項を含めること。
 *
 *	- バイナリ形式で再頒布する場合、頒布物に付属のドキュメント等の
 *		資料に、上記の著作権表示、本条件一覧、および下記免責条項を
 *		含めること。
 *
 *	- 書面による特別の許可なしに、本ソフトウェアから派生した製品の
 *		宣伝または販売促進に、本ソフトウェアの著作権者の名前または
 *		コントリビューターの名前を使用してはならない。
 *
 *	本ソフトウェアは、著作権者およびコントリビューターによって「現
 *	状のまま」提供されており、明示黙示を問わず、商業的な使用可能性、
 *	および特定の目的に対する適合性に関す暗黙の保証も含め、またそれ
 *	に限定されない、いかなる保証もないものとします。著作権者もコン
 *	トリビューターも、事由のいかんを問わず、損害発生の原因いかんを
 *	問わず、かつ責任の根拠が契約であるか厳格責任であるか(過失その
 *	他の)不法行為であるかを問わず、仮にそのような損害が発生する可
 *	能性を知らされていたとしても、本ソフトウェアの使用によって発生
 *	した(代替品または代用サービスの調達、使用の喪失、データの喪失、
 *	利益の喪失、業務の中断も含め、またそれに限定されない)直接損害、
 *	間接損害、偶発的な損害、特別損害、懲罰的損害、または結果損害に
 *	ついて、一切責任を負わないものとします。
 *
 ****************************************************************************/

#define FK_DEF_SIZETYPE
#include <FK/Surface.h>

using namespace std;

fk_Surface::fk_Surface(void)
{
	div = -1;
	changeFlg = true;
	posCache.clear();
	normCache.clear();

	return;
}

fk_Surface::~fk_Surface(void)
{
	return;
}

vector<fk_Vector> * fk_Surface::getPosCache(void)
{
	return &posCache;
}

vector<fk_Vector> * fk_Surface::getNormCache(void)
{
	return &normCache;
}

void fk_Surface::setDiv(int argDiv)
{
	if(argDiv < 1) return;

	if(argDiv != div) {
		changeFlg = true;
		div = argDiv;
	}
	return;
}

int fk_Surface::getDiv(void)
{
	return div;
}

fk_Vector fk_Surface::norm(double argU, double argV)
{
	fk_Vector		uVec, vVec, nVec;

	uVec = uDeriv(argU, argV);
	vVec = vDeriv(argU, argV);
	nVec = uVec ^ vVec;
	if(nVec.normalize() == false) {
		nVec.set(0.0, 0.0, 0.0);
	}

	return nVec;
}

void fk_Surface::makeCache(bool argSmoothFlg)
{
	if(div <= 0) return;

	makePosCache();
	makeNormCache(argSmoothFlg);
	changeFlg = false;
	return;
}

void fk_Surface::makePosCache(void)
{
	double		u, v;
	int			i, j;

	if(changeFlg == false) return;

	posCache.clear();

	for(i = 0; i <= div; i++) {
		v = double(i)/double(div);
		for(j = 0; j <= div; j++) {
			u = static_cast<double>(j)/static_cast<double>(div);
			posCache.push_back(pos(u, v));
		}
	}

	return;
}

void fk_Surface::makeNormCache(bool argSmoothFlg)
{
	int			i, j;
	fk_Vector	tmpV1, tmpV2, tmpV3, tmpNorm;

	if(changeFlg == false && smoothFlg == argSmoothFlg) return;

	normCache.clear();

	if(smoothFlg == true) {
		for(i = 0; i <= div; i++) {
			double v = static_cast<double>(i)/static_cast<double>(div);
			for(j = 0; j <= div; j++) {
				double u = static_cast<double>(j)/static_cast<double>(div);
				normCache.push_back(norm(u, v));
			}
		}
	} else {
		for(i = 0; i < div; i++) {
			for(j = 0; j < div; j++) {
				tmpV1 = posCache[static_cast<_st>(i*(div+1)+j)];
				tmpV2 = posCache[static_cast<_st>(i*(div+1)+j+1)];
				tmpV3 = posCache[static_cast<_st>((i+1)*(div+1)+j)];
				tmpNorm = (tmpV2 - tmpV1) ^ (tmpV3 - tmpV1);
				tmpNorm.normalize();
				normCache.push_back(tmpNorm);
			}
		}
	}

	return;
}
